<?php
namespace AweBooking\Payment\Controllers;

use Awethemes\Http\Request;
use AweBooking\Model\Booking;
use AweBooking\Frontend\Checkout\Checkout;
use AweBooking\Payment\Gateways\Omnipay_Gateway;
use Awethemes\Http\Exception\BadRequestException;

class Payment_Controller {
	/**
	 * The checkout instance.
	 *
	 * @var \AweBooking\Frontend\Checkout\Checkout
	 */
	protected $checkout;

	/**
	 * Constructor.
	 *
	 * @param \AweBooking\Frontend\Checkout\Checkout $checkout The checkout instance.
	 */
	public function __construct( Checkout $checkout ) {
		$this->checkout = $checkout;
	}

	/**
	 * Handle complete the purchase.
	 *
	 * @param \Awethemes\Http\Request   $request The request instance.
	 * @param string                    $gateway The gateway name.
	 * @param \AweBooking\Model\Booking $booking The booking instance.
	 *
	 * @return mixed
	 */
	public function complete( Request $request, $gateway, Booking $booking ) {
		$this->prepare_request( $booking,
			$gateway = abrs_payment_gateways()->get( $gateway )
		);

		// Gets the return url (thank you page).
		$return_url = $gateway->get_return_url( $booking );

		try {
			$response = $gateway->complete( $booking );
		} catch ( \Exception $e ) {
			abrs_add_notice( $e->getMessage(), 'error' )->important();
			return abrs_redirector()->to( add_query_arg( 'error', 1, $return_url ) );
		}

		return abrs_redirector()->to( $return_url );
	}

	/**
	 * Handle cancel the payment.
	 *
	 * @param \Awethemes\Http\Request   $request The request instance.
	 * @param \AweBooking\Model\Booking $booking The booking instance.
	 *
	 * @return mixed
	 */
	public function cancel( Request $request, Booking $booking ) {
		// Check valid hash.
		// TODO: ...

		$booking->update_status( 'cancelled' );

		return abrs_redirector()->to( abrs_get_checkout_url() );
	}

	/**
	 * Prepare checking the request.
	 *
	 * @param \AweBooking\Model\Booking $booking The booking instance.
	 * @param mixed                     $gateway The gateway instance.
	 *
	 * @throws BadRequestException
	 */
	protected function prepare_request( $booking, $gateway ) {
		if ( ! $gateway || ! $gateway instanceof Omnipay_Gateway ) {
			throw new BadRequestException( esc_html__( 'That payment method is unsupported. Please choose another one.', 'awebooking-payment' ) );
		}

		if ( 'trash' === $booking->get_status() ) {
			throw new BadRequestException( esc_html__( 'Payment could not be processed. Please try again.', 'awebooking-payment' ) );
		}
	}
}
