<?php
namespace AweBooking\Payment\Gateways;

use Omnipay\Omnipay;
use Awethemes\Http\Request;
use AweBooking\Model\Booking;
use AweBooking\Model\Booking\Payment_Item;
use AweBooking\Gateway\Gateway;
use AweBooking\Gateway\Response;

abstract class Omnipay_Gateway extends Gateway {
	/**
	 * The omnipay gateway instance.
	 *
	 * @var \Omnipay\Common\GatewayInterface
	 */
	protected $gateway;

	/**
	 * Gets the omnipay gateway instance.
	 *
	 * @return \Omnipay\Common\GatewayInterface
	 *
	 * @throws \RuntimeException
	 */
	public function get_gateway() {
		if ( is_null( $this->gateway ) ) {
			throw new \RuntimeException( 'Please setup the gateway before call this method.' );
		}

		return $this->gateway;
	}

	/**
	 * Create the gateway.
	 *
	 * @param string $gateway    The omnipay gateway.
	 * @param array  $parameters the omnipay gateway parameters.
	 *
	 * @return \Omnipay\Common\GatewayInterface
	 */
	protected function create_gateway( $gateway, array $parameters ) {
		$gateway = Omnipay::create( $gateway );

		$gateway->initialize( $parameters );

		return $gateway;
	}

	/**
	 * {@inheritdoc}
	 *
	 * @throws \RuntimeException
	 */
	public function process( Booking $booking, Request $request ) {
		$parameters = array_merge([
			'amount'    => $booking->get( 'total' ),
			'currency'  => $booking->get( 'currency' ),
			'returnUrl' => $this->get_complete_purchase_url( $booking ),
			'cancelUrl' => $this->get_cancel_purchase_url( $booking ),
		], $this->get_purchase_parameters( $booking, $request ) );

		/* @var \Omnipay\Common\Message\AbstractResponse $response */
		$response = $this->get_gateway()->purchase( $parameters )->send();

		// Response successfull.
		if ( $response->isSuccessful() ) {
			// Mark the booking is complete.
			$this->complete_booking( $booking, $response );

			return new Response( 'success', false, $booking );
		} elseif ( $response->isRedirect() ) {
			// Redirect to payment, before that
			// we will update the booking status to "processing".
			$booking->update_status( $this->get_processing_status() );

			return new Response( 'success', $response->getRedirectUrl(), $booking );
		}

		throw new \RuntimeException( $this->method_title . ': ' . $response->getMessage() );
	}

	/**
	 * Complete the purchase.
	 *
	 * @param  \AweBooking\Model\Booking $booking The booking instance.
	 * @return void
	 *
	 * @throws \RuntimeException
	 */
	public function complete( Booking $booking ) {
		/* @var $response \Omnipay\Common\Message\AbstractResponse */
		$response = $this->get_gateway()->completePurchase([
			'amount'   => $booking->get( 'total' ),
			'currency' => $booking->get( 'currency' ),
		])->send();

		// If redirect repsonse, just redirect immediately.
		if ( $response->isRedirect() ) {
			$response->redirect();
			exit;
		}

		// Throw the error message is not success.
		if ( ! $response->isSuccessful() ) {
			$error_message = esc_html__( 'Sorry, there was an error processing your payment.', 'awebooking-payment' );
			throw new \RuntimeException( "<p>{$error_message}</p><p>{$response->getMessage()}</p>" );
		}

		$this->complete_booking( $booking, $response );
	}

	/**
	 * Complete the booking.
	 *
	 * @param  \AweBooking\Model\Booking                $booking  The booking instance.
	 * @param  \Omnipay\Common\Message\AbstractResponse $response The gateway response.
	 * @return bool
	 */
	protected function complete_booking( Booking $booking, $response ) {
		// Update the booking status.
		$booking->update_status( $this->get_completed_status() );

		// Create the payment item, store the log data.
		$this->create_payment_item( $booking, $response );

		// Flush the session.
		abrs_session()->remove( 'booking_awaiting_payment' );
		abrs_reservation()->flush();

		return true;
	}

	/**
	 * Create the payment item.
	 *
	 * @param  \AweBooking\Model\Booking                $booking  The booking instance.
	 * @param  \Omnipay\Common\Message\AbstractResponse $response The gateway response.
	 */
	protected function create_payment_item( Booking $booking, $response ) {
		$payment_item = abrs_get_last_payment( $booking );

		// Check the last payment to avoid duplicate items.
		if ( ! $payment_item ) {
			$payment_item = ( new Payment_Item )->fill([
				'method'     => $this->get_method(),
				'amount'     => $booking->get( 'total' ), // TODO: ...
				'booking_id' => $booking->get_id(),
				'is_deposit' => 'off',
			]);
		}

		// Sets the transaction ID & log.
		$payment_item['transaction_id']  = $response->getTransactionReference();
		$payment_item['transaction_log'] = $response->getData();

		$payment_item->save();
	}

	/**
	 * Returns the process status.
	 *
	 * @return string
	 */
	public function get_completed_status() {
		return apply_filters( 'abrs_booking_status_payment_completed', 'completed', $this->get_method() );
	}

	/**
	 * Returns the processing status.
	 *
	 * @return string
	 */
	public function get_processing_status() {
		return apply_filters( 'abrs_booking_status_payment_processing', 'inprocess', $this->get_method() );
	}

	/**
	 * Returns the complete purchase url.
	 *
	 * @param \AweBooking\Model\Booking|int $booking The booking instance.
	 * @return string
	 */
	public function get_complete_purchase_url( $booking ) {
		$booking = abrs_get_booking( $booking );

		$purchase_url = abrs_url()->route( "/payment/complete/{$this->get_method()}/{$booking->get_id()}" );

		return apply_filters( 'abrs_get_complete_purchase_url', $purchase_url, $this );
	}

	/**
	 * Returns the complete purchase url.
	 *
	 * @param \AweBooking\Model\Booking|int $booking The booking instance.
	 * @return string
	 */
	public function get_cancel_purchase_url( $booking ) {
		$booking = abrs_get_booking( $booking );

		$purchase_url = abrs_url()->route( "/payment/cancel/{$booking->get_id()}" );

		return apply_filters( 'abrs_get_cancel_purchase_url', $purchase_url, $this );
	}

	/**
	 * Gets the extra purchase parameters.
	 *
	 * @param  \AweBooking\Model\Booking $booking The booking instance.
	 * @param  \Awethemes\Http\Request   $request The http request.
	 *
	 * @return array
	 */
	protected function get_purchase_parameters( Booking $booking, Request $request ) {
		return [];
	}
}
