<?php
namespace AweBooking\Payment\Gateways;

use Awethemes\Http\Request;
use AweBooking\Model\Booking;

class Stripe_Gateway extends Omnipay_Gateway {
	/**
	 * The gateway unique ID.
	 *
	 * @var string
	 */
	protected $method = 'stripe';

	/**
	 * The extra metadata this gateway support.
	 *
	 * @var array
	 */
	public $supports = [ 'transaction_id' ];

	/**
	 * Constructor.
	 */
	public function __construct() {
		$this->method_title = esc_html__( 'Stripe', 'awebooking-payment' );
	}

	/**
	 * Print the payment fields in the front-end.
	 *
	 * @return void
	 */
	public function display_fields() {
		parent::display_fields();
		?>

		<script src="https://js.stripe.com/v3/"></script>
		<input type="hidden" name="stripe_token">

		<div id="card-element"></div>
		<div id="card-errors" role="alert"></div>

		<style>
			.StripeElement {
				background-color: white;
				height: 40px;
				padding: 10px 12px;
				border-radius: 4px;
				border: 1px solid transparent;
				box-shadow: 0 1px 3px 0 #e6ebf1;
				-webkit-transition: box-shadow 150ms ease;
				transition: box-shadow 150ms ease;
			}

			.StripeElement--focus {
				box-shadow: 0 1px 3px 0 #cfd7df;
			}

			.StripeElement--invalid {
				border-color: #fa755a;
			}

			.StripeElement--webkit-autofill {
				background-color: #fefde5 !important;
			}

		</style>

		<script>
		(function ($) {
			var stripe = Stripe('<?php echo esc_attr( $this->get_option( 'publishable_key' ) ); ?>');
			var elements = stripe.elements();

			var card = elements.create('card');
			card.mount('#card-element');

			card.addEventListener('change', function(event) {
				var displayError = document.getElementById('card-errors');
				if (event.error) {
					displayError.textContent = event.error.message;
				} else {
					displayError.textContent = '';
				}
			});

			var form = document.getElementById('checkout-form');

			form.addEventListener('submit', function(e) {
				e.preventDefault();

				stripe.createToken(card).then(function(result) {
					if (result.error) {
						var errorElement = document.getElementById('card-errors');
						errorElement.textContent = result.error.message;
					} else {
						$('input[name="stripe_token"]').val(result.token.id);
						$(form).submit();
					}
				});
			});
		})(jQuery);
		</script>

		<?php
	}

	/**
	 * {@inheritdoc}
	 */
	protected function get_purchase_parameters( Booking $booking, Request $request ) {
		return [ 'token' => $request->get( 'stripe_token' ) ];
	}

	/**
	 * {@inheritdoc}
	 */
	public function setup() {
		$this->setting_fields();

		$this->enabled     = $this->get_option( 'enabled' );
		$this->title       = esc_html( $this->get_option( 'title' ) );
		$this->description = esc_textarea( $this->get_option( 'description' ) );

		$this->gateway = $this->create_gateway( 'Stripe', [
			'apiKey' => $this->get_option( 'secret_key' ),
		]);
	}

	/**
	 * Set the gateway settings fields.
	 *
	 * @return void
	 */
	protected function setting_fields() {
		$this->setting_fields = [
			'enabled' => [
				'name'    => esc_html__( 'Enable / Disable', 'awebooking-payment' ),
				'type'    => 'toggle',
				'default' => 'on',
			],
			'title' => [
				'name'        => esc_html__( 'Title', 'awebooking-payment' ),
				'type'        => 'text',
				'description' => esc_html__( 'This controls the title which the user sees during checkout.', 'awebooking-payment' ),
				'default'     => _x( 'Stripe', 'Stripe payment method', 'awebooking-payment' ),
			],
			'description' => [
				'name'        => esc_html__( 'Description', 'awebooking-payment' ),
				'type'        => 'textarea',
				'description' => esc_html__( 'Payment method description that the customer will see on your checkout.', 'awebooking-payment' ),
				'default'     => esc_html__( 'Pay via Stripe.', 'awebooking-payment' ),
			],
			'publishable_key' => [
				'type' => 'text',
				'name' => esc_html__( 'Publishable key', 'awebooking-payment' ),
			],
			'secret_key' => [
				'type' => 'text',
				'name' => esc_html__( 'Secret key', 'awebooking-payment' ),
			],
		];
	}
}
